-- Author      : Skylar State

local SkyUI = SkyUI
local tinsert, tremove, twipe = table.insert, table.remove, table.wipe
local friendlySpells, resSpells, longEnemySpells, enemySpells, petSpells = {}, {}, {}, {}, {}

local function AddSpell(table, spellID)
  local name = GetSpellInfo(spellID)
  if name then
    local usable, nomana = IsUsableSpell(name)
    if usable or nomana then
      table[#table + 1] = name
    end
  end
end

local _,class = UnitClass("player")
local function UpdateSpellList()
  twipe(friendlySpells)
  twipe(resSpells)
  twipe(longEnemySpells)
  twipe(enemySpells)
  twipe(petSpells)
  
  
  if class == "PRIEST" then
		AddSpell(enemySpells, 585) -- Smite (40 yards)
		AddSpell(enemySpells, 589) -- Shadow Word: Pain (40 yards)
		AddSpell(friendlySpells, 2061) -- Flash Heal (40 yards)
		AddSpell(friendlySpells, 17) -- Power Word: Shield (40 yards)
		AddSpell(resSpells, 2006) -- Resurrection (40 yards)
	elseif class == "DRUID" then
		AddSpell(enemySpells, 339) -- Entangling Roots (35 yards)
		AddSpell(longEnemySpells, 8921) -- Moonfire (40 yards)
		AddSpell(friendlySpells, 2782) -- Remove Corruption (Balance/Feral/Guardian) (40 yards)
		AddSpell(friendlySpells, 88423) -- Nature's Cure (Resto) (40 yards)
		AddSpell(resSpells, 50769) -- Revive (40 yards)
		AddSpell(resSpells, 20484) -- Rebirth (40 yards)
	elseif class == "PALADIN" then
		AddSpell(enemySpells, 20271) -- Judgement (30 yards)
		AddSpell(longEnemySpells, 20473) -- Holy Shock (40 yards)
		AddSpell(friendlySpells, 19750) -- Flash of Light (40 yards)
		AddSpell(resSpells, 7328) -- Redemption (40 yards)
	elseif class == "SHAMAN" then
		AddSpell(enemySpells, 188196) -- Lightning Bolt (Elemental) (40 yards)
		AddSpell(enemySpells, 187837) -- Lightning Bolt (Enhancement) (40 yards)
		AddSpell(enemySpells, 403) -- Lightning Bolt (Resto) (40 yards)
		AddSpell(friendlySpells, 8004) -- Healing Surge (Resto/Elemental) (40 yards)
		AddSpell(friendlySpells, 188070) -- Healing Surge (Enhancement) (40 yards)
		AddSpell(resSpells, 2008) -- Ancestral Spirit (40 yards) 
	elseif class == "WARLOCK" then
		AddSpell(enemySpells, 5782) -- Fear (30 yards)
		AddSpell(longEnemySpells, 689) -- Drain Life (40 yards)
		AddSpell(longEnemySpells, 234153) -- Drain Life (40 yards)
		AddSpell(petSpells, 755) -- Health Funnel (45 yards)
		AddSpell(friendlySpells, 20707) -- Soulstone (40 yards)
	elseif class == "MAGE" then
		AddSpell(enemySpells, 118) -- Polymorph (30 yards)
		AddSpell(longEnemySpells, 116) -- Frostbolt (Frost) (40 yards)
		AddSpell(longEnemySpells, 44425) -- Arcane Barrage (Arcane) (40 yards)
		AddSpell(longEnemySpells, 133) -- Fireball (Fire) (40 yards)
		AddSpell(friendlySpells, 130) -- Slow Fall (40 yards)
	elseif class == "HUNTER" then
		AddSpell(petSpells, 982) -- Mend Pet (45 yards)
		AddSpell(enemySpells, 75) -- Auto Shot (40 yards)
	elseif class == "DEATHKNIGHT" then
		AddSpell(enemySpells, 49576) -- Death Grip
		AddSpell(longEnemySpells, 47541) -- Death Coil (Unholy) (40 yards)
		AddSpell(resSpells, 61999) -- Raise Ally (40 yards)
	elseif class == "ROGUE" then
		AddSpell(enemySpells, 185565) -- Poisoned Knife (Assassination) (30 yards)
		AddSpell(enemySpells, 185763) -- Pistol Shot (Outlaw) (20 yards)
		AddSpell(enemySpells, 114014) -- Shuriken Toss (Sublety) (30 yards)
		AddSpell(enemySpells, 1725) -- Distract (30 yards)
		AddSpell(friendlySpells, 57934) -- Tricks of the Trade (100 yards)
	elseif class == "WARRIOR" then
		AddSpell(enemySpells, 5246) -- Intimidating Shout (Arms/Fury) (8 yards)
		AddSpell(enemySpells, 100) -- Charge (Arms/Fury) (8-25 yards)
		AddSpell(longEnemySpells, 355) -- Taunt (30 yards)
	elseif class == "MONK" then
		AddSpell(enemySpells, 115546) -- Provoke (30 yards)
		AddSpell(longEnemySpells, 117952) -- Crackling Jade Lightning (40 yards)
		AddSpell(friendlySpells, 116694) -- Effuse (40 yards)
		AddSpell(resSpells, 115178) -- Resuscitate (40 yards)
	elseif class == "DEMONHUNTER" then
		AddSpell(enemySpells, 183752) -- Consume Magic (20 yards)
		AddSpell(longEnemySpells, 185123) -- Throw Glaive (Havoc) (30 yards)
		AddSpell(longEnemySpells, 204021) -- Fiery Brand (Vengeance) (30 yards)
	end
  
end

local function getUnit(unit)
 if not unit:find("party") or not unit:find("raid") then
   for i=1, 4 do if UnitIsUnit(unit, "party"..i) then return "party"..i end end
   for i=1, 40 do if UnitIsUnit(unit, "raid"..i) then return "raid"..i end end
 else return unit end
end

SkyUI.friendlyIsInRange = function(unit)
  --Inspect (28 yards) and same phase as you
  if CheckInteractDistance(unit, 2) and UnitInPhase(unit) then return true end

  if UnitIsDeadOrGhost(unit) and #resSpells > 0 then
   for _, name in ipairs(resSpells) do if IsSpellInRange(name, unit) == 1 then return true end end
   return false
  end

  if #friendlySpells == 0 and (UnitInRaid(unit) or UnitInParty(unit)) then
   unit = getUnit(unit)
   return unit and UnitInRange(unit)
  else for _, name in ipairs(friendlySpells) do if IsSpellInRange(name, unit) == 1 then return true end end end
  return false
end


SkyUI.petIsInRange = function(unit)
  if CheckInteractDistance(unit, 2) then return true end
  for _, name in ipairs(friendlySpells) do if IsSpellInRange(name, unit) == 1 then return true end end
  for _, name in ipairs(petSpells) do if IsSpellInRange(name, unit) == 1 then return true end end
  return false
end

SkyUI.enemyIsInRange = function(unit)
  if CheckInteractDistance(unit, 2) then return true end
  for _, name in ipairs(enemySpells) do if IsSpellInRange(name, unit) == 1 then return true end end
  return false
end

SkyUI.enemyIsInLongRange = function(unit)
  for _, name in ipairs(longEnemySpells) do if IsSpellInRange(name, unit) == 1 then return true end end
  return false
end


local evt = CreateFrame("Frame") -- Event Catcher Frame

local function EventManager(self, evt, ...) if evt == "PLAYER_ENTERING_WORLD" or evt == "LEARNED_SPELL_IN_TAB" then UpdateSpellList() end end
evt:RegisterEvent("PLAYER_ENTERING_WORLD")
evt:RegisterEvent("LEARNED_SPELL_IN_TAB")
evt:SetScript("OnEvent", EventManager)



SkyUI.dumpvar = function(data)
    -- cache of tables already printed, to avoid infinite recursive loops
    local tablecache = {}
    local buffer = ""
    local padder = "    "
 
    local function _dumpvar(d, depth)
        local t = type(d)
        local str = tostring(d)
        if (t == "table") then
            if (tablecache[str]) then
                -- table already dumped before, so we dont
                -- dump it again, just mention it
                buffer = buffer.."<"..str..">\n"
            else
                tablecache[str] = (tablecache[str] or 0) + 1
                buffer = buffer.."("..str..") {\n"
                for k, v in pairs(d) do
                    buffer = buffer..string.rep(padder, depth+1).."["..k.."] => "
                    _dumpvar(v, depth+1)
                end
                buffer = buffer..string.rep(padder, depth).."}\n"
            end
        elseif (t == "number") then
            buffer = buffer.."("..t..") "..str.."\n"
        else
            buffer = buffer.."("..t..") \""..str.."\"\n"
        end
    end
    _dumpvar(data, 0)
    return buffer
end
